#!/usr/local/bin/perl -w

if ($#ARGV < 0) {
    print "Usage: perl jndi-rename.pl <directory name>\n";
    exit();
}

print "Warning: this script may corrupt these files.\n";
print "Have you made a backup of these files? ";
$ans = <STDIN>;
if ($ans !~ /^[Yy]/) {
    print "\nPlease make a backup of these files before running this script.\n";
    exit();
}


&find($ARGV[0]);

$name = "";
# $name holds the relative pathname
# $_ holds just the filename.
# The context is in the directory of $_.
sub wanted {
    if (!/\.java$/) {
        return;
    }
    if ($name =~ /SCCS/) {
	return;
    }
    print $name ."\n";
    $filename = $_;

    open (INPUT, "$filename") || die "Could not open file $_ : $!\n";
    open (OUTPUT, ">$filename.bak") || die "Could not open file $filename : $!\n";

EXIT:
    $linenum = 1;
    while (<INPUT>) {
        $count = 0;

#properties
        $count += s/jndi\.initialContextFactory/java.naming.factory.initial/g;
        $count += s/jndi\.objectFactories/java.naming.factory.object/g;
        $count += s/jndi\.urlfactory\.pkgs/java.naming.factory.url.pkgs/g;
        $count += s/jndi\.service\.url/java.naming.provider.url/g;
        $count += s/jndi\.dns\.url/java.naming.provider.dns.url/g;
        $count += s/jndi\.service\.authoritative/java.naming.authoritative/g;
        $count += s/jndi\.service\.batchsize/java.naming.batchsize/g;
        $count += s/jndi\.service\.followReferrals/java.naming.referral/g;
        $count += s/jndi\.security/java.naming.security/g;
        $count += s/jndi\.service\.language/java.naming.language/g;
#DSContext 
        $count += s/InitialDSContext/InitialDirContext/g;
        $count += s/getContinuationDSContext/getContinuationDirContext/g;
        $count += s/DSContext/DirContext/g;
        $count += s/DELETE_ATTRIBUTE/REMOVE_ATTRIBUTE/g;
#AttributeSet
        $count += s/AttributeSet/Attributes/g;
        $count += s/getAttributeCount/size/g;
        $count += s/getAttributeIds/getIDs/g;
#conflicts with DirContext getAttributes=getAll
#Attribute
        $count += s/getAttributeId/getID/g;
        $count += s/getValueCount/size/g;
        $count += s/getValues/getAll/g;
        $count += s/getValue/get/g;
        $count += s/addValue/add/g;
        $count += s/replaceValue/\/\* clear first \*\/ add/g;
        $count += s/removeValue/remove/g;
        $count += s/removeAllValues/clear/g;
#Name-related
        $count += s/getComponentCount/size/g;
        $count += s/getComponents/getAll/g;
        $count += s/getComponent/get/g;
        $count += s/isPrefix/startsWith/g;
        $count += s/isSuffix/endsWith/g;
        $count += s/prependName\(/addAll\(0\, /g;
        $count += s/appendName/addAll/g;
        $count += s/insertName/addAll/g;
        $count += s/prependComponent\(/addAll\(0\, /g;
        $count += s/appendComponent/add/g;
        $count += s/insertComponent/add/g;
        $count += s/deleteComponent/remove/g;
#Reference
        $count += s/getAddressContents/getContent/g;
        $count += s/getAddressCount/size/g;
        $count += s/getAddress/get/g;
        $count += s/prependAddr\(/add\(0\, /g;
        $count += s/appendAddr/add/g;
        $count += s/insertAddr/add/g;
        $count += s/deleteAddr/remove/g;
        $count += s/deleteAllAddresses/clear/g;
#Enumerations
        $count += s/AttributeEnumeration/NamingEnumeration/g;
        $count += s/NameClassEnumeration/NamingEnumeration/g;
        $count += s/BindingEnumeration/NamingEnumeration/g;
        $count += s/SearchEnumeration/NamingEnumeration/g;
        $count += s/RefAddrEnumeration/Enumeration/g;
        $count += s/StringEnumeration/Enumeration/g;
#
        $count += s/SearchConstraints/SearchControls/g;
	$count += s/InvalidSearchConstraintsException/InvalidSearchControlsException/g;
        $count += s/InvalidAttributeSetException/InvalidAttributesException/g;
#
        $count += s/ObjectFactoryProperty/PROP_FACTORY_OBJECT/g;
        $count += s/InitialContextFactoryProperty/PROP_FACTORY_INITIAL/g;
        $count += s/PkgPathProperty/PROP_FACTORY_URL_PKGS/g;
#toolkit
        $count += s/AtomicDSContext/AtomicDirContext/g;
        $count += s/ComponentDSContext/ComponentDirContext/g;
        $count += s/PartialCompositeDSContext/PartialCompositeDirContext/g;
        $count += s/HierMemDSCtx/HierMemDirCtx/g;
        $count += s/DSSearch/DirSearch/g;
# mod enum
        $count += s/ModificationEnumeration/ModificationItem\[\]/g;

        print OUTPUT $_;

        if ($count > 0) {
            print "$filename($linenum): $_";
        }
        $linenum++;
    }
    close (INPUT);
    close (OUTPUT);
    chmod 0666, $filename;
    rename("$filename.bak", "$filename") || die "Could not rename";
}
 

$topino = 0;
$topdev = 0;
$topnlink = 0;
$topmode = 0;

sub find {
# For Windows, use cwd() instead of pwd
#    $cwd = cwd();
    $cwd = `pwd`;
    foreach $topdir (@_) {
	(($topdev,$topino,$topmode,$topnlink) = stat($topdir))
	  || (warn("Can't stat $topdir: $!\n"), next);
	if (-d _) {
	    if (chdir($topdir)) {
		($dir,$_) = ($topdir,'.');
		$name = $topdir;
		&wanted;
		$topdir =~ s,/$,, ;
		&finddir($topdir,$topnlink + 3);
	    }
	    else {
		warn "Can't cd to $topdir: $!\n";
	    }
	}
	else {
	    unless (($dir,$_) = $topdir =~ m#^(.*/)(.*)$#) {
		($dir,$_) = ('.', $topdir);
	    }
	    $name = $topdir;
	    chdir $dir && &wanted;
	}
	chdir $cwd; 
    }
}

sub finddir {
    local($dir,$nlink) = @_;
    local($dev,$ino,$mode,$subcount);
    local($name);

    # Get the list of files in the current directory.

    opendir(DIR,'.') || (warn "Can't open $dir: $!\n", return);
    local(@filenames) = readdir(DIR);

    $nlink = $#filenames + 1;
    closedir(DIR);

    if ($nlink == 2) {        # This dir has no subdirectories.
	for (@filenames) {
	    next if $_ eq '.';
	    next if $_ eq '..';
	    $name = "$dir/$_";
	    $nlink = 0;
	    &wanted;
	}
    } else {                    # This dir has subdirectories.
	$subcount = $nlink - 2;
	for (@filenames) {
$f = $_;
	    next if $_ eq '.';
	    next if $_ eq '..';
	    $nlink = $prune = 0;
	    $name = "$dir/$_";
	    &wanted;
	    if ($subcount > 0) {    # Seen all the subdirs?

		# Get link count and check for directoriness.

		($dev,$ino,$mode,$nlink) = lstat($f) unless $nlink;
		
		if (-d _) {

		    # It really is a directory, so do it recursively.

		    if (!$prune && chdir $_) {
			&finddir($name,$nlink);
			chdir '..';
		    }
		    --$subcount;
		}
	    }
	}
    }
}
